// Microsoft Visual C++ generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "windows.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE 
BEGIN
    "#include ""windows.h""\r\n"
    "\0"
END

3 TEXTINCLUDE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_LL_ICON             ICON                    "ll_icon.ico"
IDI_LCD_LL_ICON         ICON                    "icon1.ico"
IDI_LL_ICON_SMALL       ICON                    "ll_icon_small.ico"

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

SPLASHSCREEN DIALOG  32, 32, 264, 34
STYLE DS_SETFONT | DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_VISIBLE
FONT 8, "MS Sans Serif"
BEGIN
    ICON            IDI_LL_ICON,IDC_STATIC,7,7,20,20
    LTEXT           "Loading Second Life...",666,36,13,211,8
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO 
BEGIN
    "SPLASHSCREEN", DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 257
        VERTGUIDE, 36
        TOPMARGIN, 7
        BOTTOMMARGIN, 27
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Cursor
//

TOOLGRAB                CURSOR                  "lltoolgrab.cur"
TOOLLAND                CURSOR                  "lltoolland.cur"
TOOLZOOMIN              CURSOR                  "lltoolzoomin.cur"
TOOLZOOMOUT             CURSOR                  "lltoolzoomout.cur"
TOOLCREATE              CURSOR                  "lltoolcreate.cur"
ARROWDRAG               CURSOR                  "llarrowdrag.cur"
ARROW                   CURSOR                  "llarrow.cur"
NOLOCKED                CURSOR                  "llnolocked.cur"
ARROWLOCKED             CURSOR                  "llarrowlocked.cur"
GRABLOCKED              CURSOR                  "llgrablocked.cur"
TOOLROTATE              CURSOR                  "lltoolrotate.cur"
TOOLTRANSLATE           CURSOR                  "lltooltranslate.cur"
TOOLSCALE               CURSOR                  "lltoolscale.cur"
TOOLCAMERA              CURSOR                  "lltoolcamera.cur"
TOOLPAN                 CURSOR                  "lltoolpan.cur"
TOOLFOCUS               CURSOR                  "lltoolfocus.cur"
TOOLPICKOBJECT3         CURSOR                  "toolpickobject3.cur"
ARROWCOPY               CURSOR                  "arrowcop.cur"
ARROWDRAGMULTI          CURSOR                  "llarrowdragmulti.cur"
ARROWCOPYMULTI          CURSOR                  "arrowcopmulti.cur"
TOOLPIPETTE             CURSOR                  "toolpipette.cur"
TOOLPLAY                CURSOR                  "toolplay.cur"
TOOLPAUSE               CURSOR                  "toolpause.cur"
TOOLMEDIAOPEN           CURSOR                  "toolmediaopen.cur"
TOOLBUY                 CURSOR                  "toolbuy.cur"
TOOLOPEN                CURSOR                  "toolopen.cur"
TOOLSIT                 CURSOR                  "toolsit.cur"
TOOLPATHFINDING         CURSOR                  "lltoolpathfinding.cur"
TOOLPATHFINDINGPATHSTART CURSOR                 "lltoolpathfindingpathstart.cur"
TOOLPATHFINDINGPATHSTARTADD CURSOR              "lltoolpathfindingpathstartadd.cur"
TOOLPATHFINDINGPATHEND  CURSOR                  "lltoolpathfindingpathend.cur"
TOOLPATHFINDINGPATHENDADD CURSOR                "lltoolpathfindingpathendadd.cur"
TOOLNO                  CURSOR                  "llno.cur"

/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION ${VIEWER_VERSION_MAJOR},${VIEWER_VERSION_MINOR},${VIEWER_VERSION_PATCH},${VIEWER_VERSION_REVISION}
 PRODUCTVERSION ${VIEWER_VERSION_MAJOR},${VIEWER_VERSION_MINOR},${VIEWER_VERSION_PATCH},${VIEWER_VERSION_REVISION}
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "CompanyName", "Megapahit"
            VALUE "FileDescription", "Megapahit viewer"
            VALUE "FileVersion", "${VIEWER_VERSION_MAJOR}.${VIEWER_VERSION_MINOR}.${VIEWER_VERSION_PATCH}.${VIEWER_VERSION_REVISION}"
            VALUE "InternalName", "Megapahit"
            VALUE "LegalCopyright", "Copyright (c) 2026, Megapahit."
            VALUE "OriginalFilename", "Megapahit.exe"
            VALUE "ProductName", "Megapahit viewer"
            VALUE "ProductVersion", "${VIEWER_VERSION_MAJOR}.${VIEWER_VERSION_MINOR}.${VIEWER_VERSION_PATCH}.${VIEWER_VERSION_REVISION}"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

