/**
 * @file resource.h
 *
 * $LicenseInfo:firstyear=2001&license=viewerlgpl$
 * Second Life Viewer Source Code
 * Copyright (C) 2010, Linden Research, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License only.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Linden Research, Inc., 945 Battery Street, San Francisco, CA  94111  USA
 * $/LicenseInfo$
 */

//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by viewerRes.rc
//
#define IDREMOVE                        3
#define IDI_LL_ICON                     103
#define IDC_GRABHAND                    104
#define IDI_LL_ICON_SMALL               105
#define IDC_CURSOR1                     134
#define IDC_CURSOR2                     136
#define IDC_CURSOR3                     147
#define IDB_LOGIN                       149
#define IDC_CURSOR4                     153
#define IDC_CURSOR5                     154
#define IDI_LCD_LL_ICON                 157
#define IDC_CURSOR6                     158
#define IDC_STATIC                      1000
#define IDC_RADIO_56                    1000
#define IDC_RADIO_128                   1001
#define IDC_RADIO_256                   1002
#define IDC_RADIO_CUSTOM                1003
#define IDC_STATIC_LAND                 1004
#define IDC_EDIT_LAND                   1005
#define IDC_STATIC_WATER                1006
#define IDC_EDIT_WATER                  1007
#define IDC_STATIC_WIND                 1008
#define IDC_EDIT_WIND                   1009
#define IDC_STATIC_CLOUD                1010
#define IDC_EDIT_CLOUD                  1011
#define IDC_STATIC_TASK                 1012
#define IDC_EDIT_TASK                   1013
#define IDC_STATIC_OTHER                1014
#define IDC_EDIT_OTHER                  1015
#define IDC_EDIT_USERNAME               1018
#define IDC_EDIT_PASSWORD               1019
#define IDC_EDIT_ACCOUNT                1020
#define IDC_EDIT_GUEST                  1021
#define IDC_EDIT_LASTNAME               1021
#define IDC_LABEL_TEXTURE               1028
#define IDC_BUTTON_APPLY                1030
#define IDAPPLY                         1030
#define IDC_BUTTON_GRAB                 1031
#define IDC_COMBO_TEXTURE               1032
#define IDC_RADIO_POSITION              1033
#define IDC_RADIO_SIZE                  1034
#define IDC_RADIO_ROTATION              1035
#define IDC_STATIC_POS_X                1036
#define IDC_STATIC_POS_Y                1037
#define IDC_STATIC_POS_Z                1038
#define IDC_EDIT_POS_X                  1039
#define IDC_EDIT_POS_Y                  1040
#define IDC_CHECK_AUTO_LOGIN            1040
#define IDC_EDIT_POS_Z                  1041
#define IDC_STATIC_FILENAME             1041
#define IDC_STATIC_POS_X2               1042
#define IDC_EDIT_FILENAME               1042
#define IDC_STATIC_POS_Y2               1043
#define IDC_COMBO_FACE                  1043
#define IDC_STATIC_POS_Z2               1044
#define IDC_RADIO_TEXTURE_LOW           1044
#define IDC_COMBO_MATERIAL              1044
#define IDC_EDIT_SIZE_X                 1045
#define IDC_RADIO_TEXTURE_MEDIUM        1045
#define IDC_EDIT_SIZE_Y                 1046
#define IDC_RADIO_TEXTURE_HIGH          1046
#define IDC_EDIT_SIZE_Z                 1047
#define IDC_STATIC_POS_X3               1048
#define IDC_STATIC_POS_Y3               1049
#define IDC_STATIC_POS_Z3               1050
#define IDC_EDIT_ROT_X                  1051
#define IDC_EDIT_ROT_Y                  1052
#define IDC_EDIT_ROT_Z                  1053
#define IDC_CHECK_USE_PHYSICS           1065
#define IDC_LIST1                       1068
#define IDC_LIST_USER                   1068
#define IDC_LIST_LOCATION               1068
#define IDC_BUTTON_CHANGE               1070
#define IDC_BUTTON_CLOSE                1071
#define IDC_CHAT_SAY                    1072
#define IDC_STATIC_VERT_SCALE           1073
#define IDC_STATIC_SCALE                1073
#define IDC_EDIT_VERT_SCALE             1074
#define IDC_STATIC_HORIZ_SCALE          1075
#define IDC_STATIC_SCALE2               1075
#define IDC_EDIT_HORIZ_SCALE            1076
#define IDC_BUTTON_STRETCH              1077
#define IDC_BUTTON_REPEAT               1078
#define IDC_EDIT_VERT_OFFSET            1079
#define IDC_EDIT_HORIZ_OFFSET           1080
#define IDC_EDIT_ROTATION               1081
#define IDC_STATIC_KBPS1                1082
#define IDC_STATIC_KBPS2                1083
#define IDC_BTN_INTERNAL                1083
#define IDC_BUTTON_RESET                1083
#define IDC_STATIC_KBPS3                1084
#define IDC_BTN_EXTERNAL                1084
#define IDC_STATIC_KBPS4                1085
#define IDC_BTN_QUIT                    1085
#define IDC_STATIC_KBPS5                1086
#define IDC_RESOURCE_TITLE              1086
#define IDC_STATIC_KBPS6                1087
#define IDC_RESOURCE_DESC               1087
#define IDC_EDIT_SAY                    1091
#define IDC_BTN_NORMAL                  1092
#define IDC_BTN_WHISPER                 1093
#define IDC_BTN_SHOUT                   1094
#define IDC_STATIC_ACCOUNT              1102
#define IDC_STATIC_PASSWORD             1103
#define IDC_STATIC_GUEST                1104
#define IDC_STATIC_SERVER               1104
#define IDC_STATIC_SPACESERVER          1105
#define IDC_STATIC_LASTNAME             1105
#define IDC_CHECK_GUEST                 1106
#define IDC_SCRIPT_EDIT                 1107
#define IDC_LSLSYNTAX                   1108
#define IDDONE                          1109
#define IDC_LIST                        1110
#define IDC_EDIT_MESSAGE                1111
#define IDC_BUTTON_REFRESH              1112
#define IDC_STATIC_OFFSET               1113
#define IDC_STATIC_ROTATION             1115
#define IDC_STATIC_FILEPATH             1115
#define IDC_STATIC_PX                   1118
#define IDC_STATIC_PY                   1119
#define IDC_STATIC_PZ                   1120
#define IDC_STATIC_SX                   1121
#define IDC_STATIC_SY                   1122
#define IDC_STATIC_SZ                   1123
#define IDC_STATIC_RX                   1124
#define IDC_STATIC_RY                   1125
#define IDC_STATIC_RZ                   1126
#define IDC_BTN_APPLY                   1127
#define IDC_COMBO_SERVER                1131
#define IDC_STATIC_FIRSTNAME            1132
#define IDC_EDIT_FIRSTNAME              1133
#define IDC_REMEMBER_PASSWORD           1134
#define IDC_REMEMBER_PASSWORD2          1135
#define IDC_FULL_SCREEN                 1135
#define IDC_RADIO_HOME                  1136
#define IDC_RADIO_LAST                  1137
#define IDC_COMBO1                      1138
#define IDC_COMBO_FARM                  1138
#define ID_TESTMENU_TEST                40001

// Next default values for new objects
//
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        173
#define _APS_NEXT_COMMAND_VALUE         40002
#define _APS_NEXT_CONTROL_VALUE         1139
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
